﻿Type=Class
Version=2.71
@EndOfDesignText@
'Class module
Sub Class_Globals
	Type Node(Key As String, Value As String, ParentNode As Node, ChildNodes As List)
	Private moRootNode As Node
	Private msLastError As String  = ""
	Private miNodeLevel As Int = 0 
End Sub

#Region "  Public Methods "

'Initializes the class, you may supply a JSON string at this time.
Public Sub Initialize(JSONstring As String)
	ParseJSON(JSONstring)
End Sub

' Returns any error that occured during the string parse.
Public Sub LastError As String
	Return msLastError
End Sub 

' The root node of the parsed items.
Public Sub RootNode As Node
	Return moRootNode
End Sub 

' Parses the supplied JSON string filling in the RootNode property.
Public Sub ParseNewString(JSONstring As String) As Boolean
	Return ParseJSON(JSONstring)
End Sub

'  Debug method to dump the entire node list to the log window.
Public Sub ListNodesToLog()
	If moRootNode.IsInitialized = True Then
		LogColor("----- START NODE LIST -----", Colors.Blue)
		miNodeLevel = 1 
		ListNodes(moRootNode, "")
		LogColor("----- END NODE LIST -----", Colors.Blue)
	Else
		Log("Nodes are not Initialized")
	End If 
End Sub 

' Searches the supplied LIST of child Node types and returns the node VALUE for the specified KEY if found or NULL if key not found.  
' Note that this search is not recursive, it will only search the nodes supplied, not their ChildNodes.
' <b>oChildNodes</b> - a LIST of NODE types.
' <b>sKeyLabel</b> - the key to search for in the node list.
' Returns the Value property of the first node found with the requsted key value.
Public Sub GetChildNodeValue(oChildNodes As List, sKeyLabel As String) As String
	If oChildNodes.IsInitialized Then
		If oChildNodes.Size > 0 Then
			Dim piIndex As Int
			For piIndex = 0 To oChildNodes.Size - 1
				Dim poNode As Node
				poNode = oChildNodes.Get(piIndex)
				If poNode.Key.Trim.ToLowerCase = sKeyLabel.Trim.ToLowerCase Then
					Return poNode.Value 
				End If
			Next
			Return Null
		Else
			Return Null
		End If 
	Else
		Return Null
	End If 
End Sub 

' Recursively searches the supplied Node's .ChildNodes to find the node that contains the KEY you have specified.  
' Note that it does not check the supplied Node's KEY property.
' <b>oStartingNode</b> - Node to search (not including this nodes KEY value).
' <b>sKeyLabel</b> - They key to seach for in the .ChildNodes list.
' Returns the first NODE with the matching KEY value if found or NULL if key was not found in the entire .ChildNodes list.
Public Sub FindChildNodeByKey(oStartingNode As Node, sKeyLabel As String) As Node
	If oStartingNode.ChildNodes.IsInitialized Then
		If oStartingNode.ChildNodes.Size > 0 Then
			' search just main sub-nodes first
			Dim piIndex As Int
			For piIndex = 0 To oStartingNode.ChildNodes.Size - 1
				Dim poNode As Node
				poNode = oStartingNode.ChildNodes.Get(piIndex)
				If poNode.Key.Trim.ToLowerCase = sKeyLabel.Trim.ToLowerCase Then
					Return poNode
				End If
			Next
			' if we get here then none of the main node text values matched check all child nodes
			For piIndex = 0 To oStartingNode.ChildNodes.Size - 1
				Dim poSubNode As Node
				poSubNode = FindChildNodeByKey(oStartingNode.ChildNodes.Get(piIndex), sKeyLabel)
				If poSubNode <> Null Then
					Return poSubNode
				End If 
			Next
			' not found
			Return Null 
		Else
			Return Null
		End If 
	Else
		Return Null
	End If
End Sub 
#End Region 


#Region "  Private Methods "

Private Sub ParseJSON(sJSON As String) As Boolean 
	Dim poJ As JSONParser 
	Dim pbResult As Boolean = False 
	
	msLastError = ""
	Dim poRootNode As Node 
	poRootNode.Initialize 
	moRootNode = poRootNode
	moRootNode.Key = "Root"
	moRootNode.ParentNode = Null

	If sJSON <> "" Then
		'check the first non-space character. [ - NextArray, { - NextObject. 
		If sJSON.Trim.StartsWith("[") Then
			' next array
			Try
				poJ.Initialize(sJSON)
				Dim poList As List 
				poList = poJ.NextArray 
				moRootNode.ChildNodes = WalkTree(poList, moRootNode)
				pbResult = True
			Catch
				msLastError = LastException
			End Try
		Else
			' next object
			Try
				poJ.Initialize(sJSON)
				Dim poMap As Map 
				poMap = poJ.NextObject 
				moRootNode.ChildNodes = WalkTree(poMap, moRootNode)
				pbResult = True
			Catch
				msLastError = LastException
			End Try 
		End If 
	End If 
	
	Return pbResult
End Sub 

Private Sub WalkTree(NextObj As Object, ParentNode As Node) As List 
	Dim poThisNode As List
	poThisNode.Initialize 

	If NextObj Is List Then
		Dim poList As List = NextObj
		Dim piIndex As Int
		For piIndex = 0 To poList.Size - 1
			Dim poObj As Object
			poObj = poList.Get(piIndex)
			Dim poNewNode As Node
			poNewNode.Initialize 
			poNewNode.Key = ParentNode.Key  & "-" & (piIndex + 1)
			poNewNode.Value = Null
			poNewNode.ParentNode = ParentNode
			poThisNode.Add(poNewNode)
			poNewNode.ChildNodes = WalkTree(poObj, poNewNode)
		Next 
	Else
		If NextObj Is Map Then
			Dim poMap As Map = NextObj
			For Each sKey As String In poMap.Keys 
				If poMap.Get(sKey) Is List OR poMap.Get(sKey) Is Map Then
					Dim poNewNode As Node
					poNewNode.Initialize 
					poNewNode.Key = sKey
					poNewNode.Value = Null
					poNewNode.ParentNode = ParentNode
					poThisNode.Add(poNewNode)
					poNewNode.ChildNodes = WalkTree(poMap.Get(sKey), poNewNode)
				Else
					Dim poNewItem As Node
					poNewItem.Initialize 
					poNewItem.Key = sKey
					poNewItem.Value = poMap.Get(sKey)
					poNewItem.ParentNode = ParentNode
					poNewItem.ChildNodes.Initialize 
					poThisNode.Add(poNewItem)
				End If 
			Next 
		Else
			' not a map or a list, must be a value?
			Dim poNewItem As Node
			poNewItem.Initialize 
			poNewItem.Key = NextObj
			poNewItem.Value = NextObj
			poNewItem.ParentNode = ParentNode
			poNewItem.ChildNodes.Initialize 
			poThisNode.Add(poNewItem)
		End If 
	End If

	Return poThisNode
End Sub 

Private Sub ListNodes(oNode As Node, sIndent As String)
	Dim piIndex As Int
	
	LogColor(sIndent & miNodeLevel & ".> " & oNode.Key & " (Val=" & oNode.Value & ")", Colors.RGB(50,50,180))
		
	For piIndex = 0 To oNode.ChildNodes.Size - 1
		miNodeLevel = miNodeLevel + 1
		ListNodes(oNode.ChildNodes.Get(piIndex), sIndent & "   ")
	Next 
	miNodeLevel = miNodeLevel - 1
End Sub 			
#End Region 

